<?php

namespace Drupal\document_ocr\Controller;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Render\FormattableMarkup;

/**
 * Document OCR processors list builder.
 *
 * @ingroup document_ocr
 */
class ProcessorListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header = [];
    $header['label'] = $this->t('Label');
    $header['description'] = $this->t('Description');
    $header['processor'] = $this->t('Processor Name');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['label'] = $entity->label();
    $row['description'] = $entity->getDescription();
    $row['processor'] = new FormattableMarkup('@name <br> @ext', [
      '@name' => $entity->getProcessorPlugin()->getName(),
      '@ext' => $this->t('<span class="form-item__description">Supported Files: @list</span>', [
        '@list' => $entity->getProcessorPlugin()->getExtensions()
      ]),
    ]);
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $ajax_attributes = [
      'class' => ['use-ajax'],
      'data-dialog-type' => 'modal',
      'data-dialog-options' => Json::encode(['width' => 620]),
    ];
    $operations = parent::getDefaultOperations($entity);
    $operations['configuration'] = [
      'title' => $this->t('Configuration'),
      'url' => $this->ensureDestination($entity->toUrl('configuration')),
      'attributes' => $ajax_attributes,
    ];
    $operations['edit'] = [
      'title' => $this->t('Edit'),
      'url' => $this->ensureDestination($entity->toUrl('edit')),
      'attributes' => $ajax_attributes,
    ];
    // Default processor should be available in the UI.
    if ($entity->id() !== 'default') {
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'url' => $this->ensureDestination($entity->toUrl('delete')),
        'attributes' => $ajax_attributes,
      ];
    }
    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build = parent::render();
    $build['table']['#empty'] = $this->t('Whoops looks like there are no processors configured.');
    return $build;
  }

}
