<?php

namespace Drupal\document_ocr\Controller;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Component\Serialization\Json;

/**
 * Document OCR transformers list builder.
 *
 * @ingroup document_ocr
 */
class TransformerListBuilder extends ConfigEntityListBuilder {

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header = [];
    $header['label'] = $this->t('Label');
    $header['description'] = $this->t('Description');
    $header['transformer'] = $this->t('Transformer Name');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['label'] = $entity->label();
    $row['description'] = $entity->getDescription();
    $row['transformer'] = $entity->getTransformerPlugin()->getName();
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $ajax_attributes = [
      'class' => ['use-ajax'],
      'data-dialog-type' => 'modal',
      'data-dialog-options' => Json::encode(['width' => 620]),
    ];
    $operations = parent::getDefaultOperations($entity);
    $operations['configuration'] = [
      'title' => $this->t('Configuration'),
      'url' => $this->ensureDestination($entity->toUrl('configuration')),
      'attributes' => $ajax_attributes,
    ];
    $operations['edit'] = [
      'title' => $this->t('Edit'),
      'url' => $this->ensureDestination($entity->toUrl('edit')),
      'attributes' => $ajax_attributes,
    ];
    // Default transformer should be available in the UI.
    if ($entity->id() !== 'default') {
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'url' => $this->ensureDestination($entity->toUrl('delete')),
        'attributes' => $ajax_attributes,
      ];
    }
    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build = parent::render();
    $build['table']['#empty'] = $this->t('Whoops looks like there are no transformers configured.
      Usually Basic transformer is configured by default but it appears like you might have deleted the config for it.
      No worries you can grab it from the module directory and re-import it.');
    return $build;
  }

}
