<?php

namespace Drupal\document_ocr\Entity\Form\Processor;

use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration entity edit form.
 */
class ConfigurationForm extends BaseForm {

  /**
   * {@inheritdoc}
   */
  protected $success_message = '%label processor configuration updated.';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'document_ocr_processor_' . $this->entity->getProcessor();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $form['#attached']['library'][] = 'document_ocr/document_ocr.admin';
    $document_ocr_processor = $this->entity->getProcessorPlugin();
    $form_state->set('document_ocr_processor', $document_ocr_processor);
    $document_ocr_processor
      ->setCredentials($this->entity->getCredentials())
      ->setConfiguration($this->entity->getConfiguration());
    $configuration_form = $document_ocr_processor->configurationForm($form, $form_state);
    if ($configuration_form !== $form) {
      $form = array_merge($form, $configuration_form);
      $form['actions']['submit']['#value'] = $this->t('Update Processor');
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function buildEntity(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\document_ocr\DocumentOcrProcessor $entity */
    $entity = parent::buildEntity($form, $form_state);
    if ($configuration = $form_state->get('document_ocr_processor')->configurationValues($form, $form_state)) {
      $entity->setConfiguration($configuration);
    }
    return $entity;
  }

}
