<?php

namespace Drupal\document_ocr\Services;

/**
 * Document OCR Batch API.
 *
 * @ingroup document_ocr
 */
class Batch {

  /**
   * Document processing.
   */
  public static function process($item, &$context) {
    try {
      if ($import = \Drupal::entityTypeManager()->getStorage('document_ocr_import')->load($item->id)) {
        if ($document = $import->getDocumentOcr()) {
          $settings = $document->getSettings();
          if ($import->getAttempts() < $settings['attempts'] && $import->isPending()) {
            if ($destination = \Drupal::service('document_ocr.process')->runPendingImport($import, $item)) {
              $context['results'][] = $item->id;
              $context['message'] = t('Processing %name.', ['%name' => $destination->label()]);
            }
            else {
              $error_message = t('Unable to process %name. It was added to the retry queue and will be processed later', ['%name' => $document->label()]);
              $context['message'] = $error_message;
              $context['results']['error'][] = $error_message;
            }
          }
          else {
            $error_message = t('Attempt limit reached or document marked as processed for %name.', ['%name' => $import->getFile()->getFilename()]);
            $context['message'] = $error_message;
            $context['results']['error'][] = $error_message;
          }
        }
      }
    }
    catch (\Exception $e) {
      $context['message'] = $e->getMessage();
      $context['results']['error'][] = $e->getMessage();
    }
  }

  /**
   * One-time files processing.
   */
  public static function processOneTime($item, &$context) {
    try {
      if ($destination = \Drupal::service('document_ocr.process')->runOneTimeImport($item)) {
        $context['results'][] = $item->file->id();
        $context['message'] = t('Processing %name.', ['%name' => $item->file->getFilename()]);
      }
      else {
        $error_message = t('Unable to process %name.', ['%name' => $item->file->getFilename()]);
        $context['message'] = $error_message;
        $context['results']['error'][] = $error_message;
      }
    }
    catch (\Exception $e) {
      $context['message'] = $e->getMessage();
      $context['results']['error'][] = $e->getMessage();
    }
  }

  /**
   * Process older entities.
   */
  public static function processOlderEntity($entity, &$context) {
    try {
      $context['results'][] = $entity->id();
      $context['message'] = t('Processing %label.', ['%label' => $entity->label()]);
      \Drupal::service('document_ocr.process')->processEntity($entity);
    }
    catch (\Exception $e) {
      $context['message'] = $e->getMessage();
      $context['results']['error'][] = $e->getMessage();
    }
  }

  /**
   * Complete callback for file process.
   */
  public static function completeProcessCallback($success, $results, $operations) {
    if (!empty($results['error'])) {
      foreach ($results['error'] as $error_message) {
        self::complete($error_message, $error_message, FALSE, $results);
      }
    }
    else {
      self::complete('1 document processed.', '@count documents processed.', $success, $results);
    }
  }

  /**
   * Helper method for complete callbacks.
   */
  protected static function complete($singular, $plural, $success, $results) {
    if ($success) {
      $message = \Drupal::translation()->formatPlural(count($results), $singular, $plural);
      \Drupal::messenger()->addMessage($message, 'status', FALSE);
      \Drupal::logger('document_ocr')->notice($message);
    }
    else {
      \Drupal::messenger()->addWarning($singular, 'warning', FALSE);
      \Drupal::logger('document_ocr')->warning($singular);
    }
  }

}
