<?php

namespace Drupal\document_ocr\Services;

use Drupal\Core\Cache\CacheBackendInterface;

/**
 * OpenAi service.
 */
class OpenAi {

  /**
   * The cache.default cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * Cache expiration in seconds.
   */
  protected $interval = 10800; // 3 hours.

  protected $client;

  /**
   * {@inheritdoc}
   */
  public function __construct(CacheBackendInterface $cache) {
    $this->cacheBackend = $cache;
  }

  /**
   * Set credentials.
   */
  public function setCredentials($credentials) {
    try {
      $org = !empty($credentials['apiorg']) ? $credentials['apiorg'] : NULL;
      if (!empty($credentials['apikey'])) {
        $this->client = \OpenAI::client($credentials['apikey'], $org);
      }
    }
    catch (\Exception $ex) {
      \Drupal::logger('document_ocr')->error($ex->getMessage());
    }
    return $this;
  }

  /**
   * Get models list.
   */
  public function getModels() {
    $list = [];
    $cid = 'openai:models:list';
    if ($cache = $this->cacheBackend->get($cid)) {
      $list = $cache->data;
    }
    else {
      try {
        $response = $this->client->models()->list();
        if (!empty($response->data)) {
          foreach ($response->data as $item) {
            $list[$item->id] = $item->id;
          }
        }
      }
      catch (\Exception $ex) {
        \Drupal::logger('document_ocr')->error($ex->getMessage());
      }
      ksort($list);
      $this->cacheBackend->set($cid, $list, time() + $this->interval, [$cid]);
    }
    return $list;
  }

  /**
   * Creates a completion for the provided prompt and parameters.
   */
  public function request($configuration) {
    try {
      $response = $this->client->completions()->create($configuration);
      return $response;
    }
    catch (\Exception $ex) {
      \Drupal::logger('document_ocr')->error($ex->getMessage());
    }
  }

}
